/*

*/

import renderer.scene.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;
import renderer.gui.*;

import java.awt.Color;
import java.awt.event.*;

/**
   This program renders a fan of lines two ways, one way using the
   rasterizer without any antialiasing, and a second way using
   the raterizer that implements a simple antialiasing algorithm.
   Use the 'a' key to switch between the two renderings.

   To see how the antialiasing works, look at the rendered
   images using a pixel magnifying program.
*/
@SuppressWarnings("serial")
public class AntialiasingDemo_2 extends InteractiveFrame
{
   private static final int numberOfLines = 30;

   private static final Color backGroundColor  = Color.green;
   private static final Color foreGroundColor1 = Color.red;
   private static final Color foreGroundColor2 = Color.red; // try making this different

   private Scene scene;

   /**
      This constructor instantiates the Scene object
      and initializes it with appropriate geometry.
   */
   public AntialiasingDemo_2(String title, int fbWidth, int fbHeight)
   {
      super(title, fbWidth, fbHeight);

      // Create the Scene object that we shall render
      scene = new Scene();

      // Create a Model object.
      Model model = new Model();

      // Add the model to the Scene.
      scene.addModel(model);

      // Create a fan of lines, coming out of (-1, -1, -1), in
      // the square with -1 <= x <= 1, -1 <= y <= 1, and z = -1.
      model.addVertex( new Vertex(-1, -1, -1) );
      model.addColor( foreGroundColor1 );
      model.addColor( foreGroundColor2 );
      for (int i = 0; i < numberOfLines; i++)
      {
         model.addVertex( new Vertex(1, -1 + 2*(double)i/(double)numberOfLines, -1) );
         model.addLineSegment(new LineSegment(0, 2*i+1, 0, 1));  // slope < 1
         model.addVertex( new Vertex(-1 + 2*(double)i/(double)numberOfLines, 1, -1) );
         model.addLineSegment(new LineSegment(0, 2*i+2, 0, 1));  // slope > 1
      }
      model.addVertex( new Vertex(1, 1, -1) );
      model.addLineSegment(new LineSegment(0, 2*numberOfLines+1, 0, 1)); // 45 degree line
   }


   // Implement part of the KeyListener interface.
   @Override public void keyTyped(KeyEvent e)
   {
      //System.out.println( e );

      char c = e.getKeyChar();
      if ('h' == c)
      {
         print_help_message();
         return;
      }
      else if ('d' == c)
      {
         Pipeline.debug = ! Pipeline.debug;
         Clip.debug = ! Clip.debug;
         RasterizeAntialias.debug = ! RasterizeAntialias.debug;
      }
      else if ('a' == c)
      {
         RasterizeAntialias.doAntialiasing = ! RasterizeAntialias.doAntialiasing;
         System.out.print("Anti-aliasing is turned ");
         System.out.println(RasterizeAntialias.doAntialiasing ? "On" : "Off");
      }
      else if ('g' == c)
      {
         RasterizeAntialias.doGamma = ! RasterizeAntialias.doGamma;
         System.out.print("Gamma correction is turned ");
         System.out.println(RasterizeAntialias.doGamma ? "On" : "Off");
      }

      // Render again.
      FrameBuffer fb = this.fbp.getFrameBuffer();
      fb.clearFB();
      Pipeline.render(scene, fb.vp);
      fbp.update();
      repaint();
   }


   // Implement part of the MouseListener interface.
   @Override public void mouseClicked(MouseEvent e)
   {
      //System.out.println( e );

      if (! e.isControlDown())
      {
         RasterizeAntialias.doAntialiasing = ! RasterizeAntialias.doAntialiasing;
         System.out.print("Anti-aliasing is turned ");
         System.out.println(RasterizeAntialias.doAntialiasing ? "On" : "Off");
      }
      else if (e.isControlDown())
      {
         RasterizeAntialias.doGamma = ! RasterizeAntialias.doGamma;
         System.out.print("Gamma correction is turned ");
         System.out.println(RasterizeAntialias.doGamma ? "On" : "Off");
      }

      // Render again.
      FrameBuffer fb = this.fbp.getFrameBuffer();
      fb.clearFB();
      Pipeline.render(scene, fb.vp);
      fbp.update();
      repaint();
   }


   // Implement part of the ComponentListener interface.
   @Override public void componentResized(ComponentEvent e)
   {
      //System.out.println( e );

      // Get the new size of the FrameBufferPanel.
      int w = this.fbp.getWidth();
      int h = this.fbp.getHeight();

      // Create a new FrameBuffer that fits the new window size.
      FrameBuffer fb = new FrameBuffer(w, h, backGroundColor);
      this.fbp.setFrameBuffer(fb);
      fb.clearFB();
      Pipeline.render(scene, fb.vp);
      fbp.update();
      repaint();
   }


   /**
      Create an instance of this class which has
      the affect of creating the GUI application.
   */
   public static void main(String[] args)
   {
      print_help_message();

      // Define initial dimensions for a FrameBuffer.
      int width  = 900;
      int height = 900;

      // We need to call the program's constructor in the
      // Java GUI Event Dispatch Thread, otherwise we get a
      // race condition between the constructor (running in
      // the main() thread) and the very first ComponentEvent
      // (running in the EDT).
      javax.swing.SwingUtilities.invokeLater(
         () -> {new AntialiasingDemo_2("Renderer 5", width, height);}
      );
   }//main()


   private static void print_help_message()
   {
      System.out.println("Use the 'd' key to toggle debugging information on and off.");
      System.out.println("Use the 'a' key to toggle antialiasing on and off.");
      System.out.println("Use the 'g' key to toggle gamma correction on and off.");
      System.out.println("Use mouse click to toggle antialiasing on and off.");
      System.out.println("Use mouse control-click to toggle gamma correction on and off.");
      System.out.println("Use the 'h' key to redisplay this help message.");
   }
}
